"use strict";

exports.__esModule = true;
exports.useContext = useContext;
exports.clearContexts = clearContexts;
exports.shim = shim;
exports._makeProvider = _makeProvider;
exports._render = _render;

var _react = _interopRequireWildcard(require("react"));

var _server = require("react-dom/server");

var _htmlReactParser = _interopRequireDefault(require("html-react-parser"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _inheritsLoose(subClass, superClass) { subClass.prototype = Object.create(superClass.prototype); subClass.prototype.constructor = subClass; subClass.__proto__ = superClass; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

// Array to store React contexts
var contexts = []; // Use specific context

function useContext(Context) {
  contexts.push(Context);
} // Clear contexts in use


function clearContexts() {
  contexts.length = 0;
} // Shim `React.createContext()` to capture contexts


var shimmed = false;

function shim() {
  if (shimmed) return;
  shimmed = true;
  var createContext = _react.default.createContext;

  _react.default.createContext = function () {
    var Context = createContext.apply(this, arguments);
    contexts.push(Context);
    return Context;
  };
} // Create Provider component for specific context types


function _makeProvider(contextTypes) {
  var _class, _temp;

  return _temp = _class =
  /*#__PURE__*/
  function (_Component) {
    _inheritsLoose(ProvideContext, _Component);

    function ProvideContext() {
      return _Component.apply(this, arguments) || this;
    }

    var _proto = ProvideContext.prototype;

    _proto.getChildContext = function getChildContext() {
      return this.props.context;
    };

    _proto.render = function render() {
      return this.props.children;
    };

    return ProvideContext;
  }(_react.Component), _defineProperty(_class, "childContextTypes", contextTypes), _temp;
} // Server-side render, propagating contexts to children


function _render(self, ProvideContext) {
  // Wrap element in provider for legacy context
  var element = _react.default.createElement(ProvideContext, {
    context: self.context
  }, self.props.children); // Wrap element in consumers and providers for new context.
  // Consumers are added outside HTML render, to capture value of all contexts.
  // Providers are added inside HTML render, to reinject context values back
  // into the rendered element.


  return stackConsumers(function (values) {
    var elementWithProviders = stackProviders(element, values);

    try {
      return (0, _htmlReactParser.default)((0, _server.renderToStaticMarkup)(elementWithProviders));
    } catch (e) {
      return _react.default.createElement("div", null, self.props.fallBack());
    }
  });
} // Create stack of consumer components


function stackConsumers(retFn) {
  var values = [];

  var fn = function fn() {
    return retFn(values);
  };

  var _loop = function _loop(i) {
    var Context = contexts[i];
    var childFn = fn;

    fn = function fn() {
      return _react.default.createElement(Context.Consumer, null, function (value) {
        values[i] = value;
        return childFn();
      });
    };
  };

  for (var i = 0; i < contexts.length; i++) {
    _loop(i);
  }

  return fn();
} // Create stack of provider components


function stackProviders(element, values) {
  for (var i = 0; i < values.length; i++) {
    var Context = contexts[i];
    var childElement = element;
    element = _react.default.createElement(Context.Provider, {
      value: values[i]
    }, childElement);
  }

  return element;
}